/*

   Each parameter has a number used as identifier. These are the following:
		[entry field]	[toggle button]	[spin button]
	0	version number
	1	fittitle
	2	nlines
	3	app_s
	4	app_b
	5	sbg
	6			sbgit
	7	dsig
	8			sigunit(radio)
	9	dlam
	29			ikev
	10			ilamit
	11			isigit
	12			iformat(radio)
	13			ires
	14			iplot
	15			ilab
	16			ierrplot
	17			ilog
	18					nbin
	19					nlik
	20			ispline
	21			idlam
	31			idsig
	22					iord
	23			bgsmooth
	24			iportrait
	25					verbose
	26			spec_only
	27			fit_only
	28	rem_index
	30			sbgchi
	32			fixflag
	33					medsig
						*/
#include <sys/types.h>
#include <sys/resource.h>
/* #include <sys/wait.h> */
#include <math.h>

#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#if defined(USE_TIMEOUT)
#include <signal.h>
#endif

#include "Cora.h"

#include "log_window.h"

char version[5];

/*      program flags    */
int ireplot=0;

int iuse_fixpt = 1;


/*      Default values   */
int nlines = 1;
int sigunit = 0, ikev=0;
gchar fittitle[MAXTEXTL] = "default session";
double lambda[MAXLINES]  =   {18.97};
double sigma[MAXLINES]   =    {0.03};
double f[MAXLINES]       =    {0.00};

double v     = 0.;
double app_s = 1.;
double app_b = 1.;
double sbg   = 0.;
double medsig = 3.;

int fixflag   = 1;
int sbgit     = 1;
int bgsmooth  = 1;
int sbgchi    = 0;
int ilamit    = 1;
int isigit    = 1;
int iformat   = 0;
int iportrait = 0;
int ires      = 0;
int iplot     = 1;
int ierrplot  = 0;
int ilab      = 0;
int ilog      = 0;
int nbin      = 1;
int spec_only = 0;
int fit_only = 0;
int nlik      = 0;
int ispline   = 0;
int idlam     = 0;
int idsig     = 0;
int rem_index = 1;

double dsig = .02;
double dlam = .50;
double iord =  1.;

char datafile[MAXPATH+1];
char conffile[MAXPATH+1];
char proffile[MAXPATH];
char  resfile[MAXPATH+1];
char  texfile[MAXPATH+1];
char plotfile[MAXPATH+1];

int iprof=0;    /* flag for eventual numerical profile =1   */
int    pro_type[MAXLINES];     /* the profile type          */
int    fit_wave[MAXLINES];     /* whether to fit wavelength */
int    fit_sigm[MAXLINES];     /* whether to fit sigme      */
int    fit_lock[MAXLINES];     /* lock lambda to previous   */
float  beta[MAXLINES];	       /* exponent in Beta model    */
int    imaxfit;                /* currently unused, but set */


int    verbose = 0;
int    quiet   = 0;

float * wave, * flux, *bb, * err;
double dt=1.;

int count;

GtkWidget *line_window;
GtkWidget *entry_arr[NENTRY];
GtkWidget *spin_arr[4];
GtkWidget *toggle[NENTRY];
GtkWidget *entry_line[NENTRY];
GtkWidget *entry_sig[NENTRY];

GtkWidget *entry_f[NENTRY];
GtkWidget *entry_beta[NENTRY];

GtkWidget *profuse;
GtkWidget *eflag_t[NENTRY];
GtkWidget *eflag_w[NENTRY];
GtkWidget *eflag_s[NENTRY];
GtkWidget *eflag_l[NENTRY];

GtkWidget *entry_v;
GtkWidget *fixtoggle;
int lines_open = 0;

#include <sys/stat.h>

/* Prototype */

GtkWidget *
create_toggle (GtkWidget * box_loop, void *enter_switch, char *callarg, 
	       char *deftext, char *labtext);


static int
test_confdir ()
{
  char path[256];
  struct stat buf;
  int mask;
  int error;

  g_snprintf(path, 255, "%s/.cora", g_get_home_dir()); 

  if (0 != (stat (path, &buf)))
    {
      error = errno;
      if (error == ENOENT)
	{
	  mask = umask (0);
	  if (-1 == mkdir (path, S_IRWXU))
	    {
	      error = errno;
	      g_warning("Error creating directory %s: %s", 
			path, g_strerror(error));
	      return (-1);
	    } 
	  umask (mask);
	  return (0);
	}
      else
	{
	  g_warning("Error accessing %s: %s", path, g_strerror(error)); 
	  return (-1);
	}
    }
  else
    return 0;
}

/*  -------------------------------------------------------------------- */
static void
save_setting (char * what, char * value)
{
  char path[256];
  FILE * fd;
  int error;

  if (what == NULL || value == NULL)
    return;

  if (0 != test_confdir())
    return;

  error = g_snprintf(path, 255, "%s/.cora/%s", g_get_home_dir(), what);
  g_warning("g_snprintf returned %d", error); /* FIXME */

  fd = fopen(path, "w");
  if (fd)
    {
      fprintf(fd, "%s\n", value);
      fclose (fd);
    }
  else
    {
      error = errno;
      g_warning("Error opening %s: %s", path, g_strerror(error));
    } 
  return;
}

static char setting[256];

char * get_setting(char * what)
{
  char path[256];
  int  i;
  FILE * fd;

  if (what == NULL)
    return NULL;

  g_snprintf(path, 255, "%s/.cora/%s", g_get_home_dir(), what);

  fd = fopen(path, "r");
  if (fd)
    {
      fgets(setting, 256, fd);
      fclose (fd);
      i = strlen(setting);
      if (setting[i-1] == '\n')
	setting[i-1] = '\0';
      return &setting[0];
    }
  else
    return NULL;

}


	

/*  -------------------------------------------------------------------- */
void 
delete_lines (GtkWidget * widget, gpointer data)
{
  lines_open = 0;
  gtk_widget_destroy (GTK_WIDGET(line_window));
}

static  int          exit_val    = -1;
static  GtkWidget   *exit_dialog;

/*  -------------------------------------------------------------------- */
void do_exit_app (GtkWidget * widget, 
                  gpointer *data)
{
  gint button_number;

  sscanf ((char *) data, "%d", &button_number);

  switch (button_number) {
  case 0: /* NO button  */
    exit_val = 0;
    break;
  case 1: /* YES button */
    exit_val = 1;
    break;
  default:
    break;
  };
  return;
}

GtkWidget   *popup_dialog;

void destroy_popup()
{
  gtk_widget_destroy(popup_dialog);
}

/*  -------------------------------------------------------------------- */
void popup_message (char * text)
{
  GtkWidget   *label;
  GtkWidget   *button;
  GtkWidget   *box1;
  GtkWidget   *hbox;
  GtkWidget   *separator;

  popup_dialog = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (popup_dialog), "Message");
  gtk_widget_set_usize (popup_dialog, 400, 90);
  gtk_window_set_position(GTK_WINDOW(popup_dialog), GTK_WIN_POS_MOUSE);
  gtk_signal_connect (GTK_OBJECT (popup_dialog), "destroy",
                      GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);

  box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (popup_dialog), box1);
  gtk_container_border_width (GTK_CONTAINER (box1), 5);

  label = gtk_label_new (text);
  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, TRUE, 5);
  gtk_widget_show (label);
  gtk_widget_show (box1);
   
  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
  gtk_widget_show (separator);    

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (box1), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

  button = gtk_button_new_with_label ("close window");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (destroy_popup), NULL);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  (GTK_WIDGET_FLAGS (button)  |= (GTK_CAN_DEFAULT));
  gtk_widget_grab_default (button); 
  gtk_widget_show (button);

  gtk_widget_show (popup_dialog);

  return;
}




/*  -------------------------------------------------------------------- */
void do_exit_confirm ()
{

  GtkWidget   *label;
  GtkWidget   *button;
  GtkWidget   *box1;
  GtkWidget   *hbox;
  GtkWidget   *separator;

  exit_dialog = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_widget_set_usize (exit_dialog, 200, 80);
  gtk_window_set_position(GTK_WINDOW(exit_dialog), GTK_WIN_POS_MOUSE);
  gtk_signal_connect (GTK_OBJECT (exit_dialog), "destroy",
                      GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);

  box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (exit_dialog), box1);
  gtk_container_border_width (GTK_CONTAINER (box1), 5);

  label = gtk_label_new ("Really exit ?");
  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, TRUE, 5);
  gtk_widget_show (label);
  gtk_widget_show (box1);
   
  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
  gtk_widget_show (separator);    

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (box1), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

  button = gtk_button_new_with_label ("Yes");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (do_exit_app), "1");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  (GTK_WIDGET_FLAGS (button)  |= (GTK_CAN_DEFAULT));
  gtk_widget_grab_default (button); 
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("No");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (do_exit_app), "0");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  (GTK_WIDGET_FLAGS (button)  |= (GTK_CAN_DEFAULT));
  gtk_widget_grab_default (button); 
  gtk_widget_show (button);

  gtk_widget_show (exit_dialog);

  return;
}

/*  -------------------------------------------------------------------- */
gint 
do_delete_event (GtkWidget * widget,
	      GdkEvent * event,
	      gpointer data)
{
  /* If you return FALSE in the "delete_event" signal handler,
   * GTK will emit the "destroy" signal. Returning TRUE means
   * you don't want the window to be destroyed.
   * This is useful for popping up 'are you sure you want to quit?'
   * type dialogs. */


  /* Change TRUE to FALSE and the main window will be destroyed with
   * a "delete_event". */

  exit_val = -1;
  do_exit_confirm ();

  do
    {
      while (gtk_events_pending()) 
	gtk_main_iteration();
    } 
  while (exit_val == -1);

  if (exit_val == 1)
    gtk_main_quit();

  gtk_widget_destroy(exit_dialog);
  return(TRUE); 
}

/*  -------------------------------------------------------------------- */
void
delete_event (GtkWidget * widget,
              GdkEvent * event,
              gpointer data)
{
  /* function calls gint function do_delete_event with proper argument list */
  do_delete_event(NULL,NULL,NULL);
  return;
}

/*  -------------------------------------------------------------------- */
void about_cb (GtkWidget * w, int data)
{

  GtkWidget   *about_dialog;
  GtkWidget   *label;
  GtkWidget   *button;
  GtkWidget   *box1;
  GtkWidget   *separator;

  about_dialog = gtk_window_new (GTK_WINDOW_DIALOG);
/*  gtk_widget_set_usize (about_dialog, 300, 120);  */
  gtk_signal_connect (GTK_OBJECT (about_dialog), "destroy",
                      GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);

  box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (about_dialog), box1);
  gtk_container_border_width (GTK_CONTAINER (box1), 5);

  label = gtk_label_new ("Cora line fitting tool\n"\
			 "(c) 2000, 2001\n"\
			 "Jan-Uwe Ness\n <jness@hs.uni-hamburg.de>\n"\
			 "Rainer Wichmann\n <rwichmann@hs.uni-hamburg.de>\n"\
			 " Version "VERSION
			 );
  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, TRUE, 5);
  gtk_widget_show (label);
  gtk_widget_show (box1);

  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
  gtk_widget_show (separator);    

  button = gtk_button_new_with_label ("Ok");
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy), 
			     GTK_OBJECT (about_dialog));
  gtk_box_pack_start (GTK_BOX (box1), button, FALSE, FALSE, 0);
  (GTK_WIDGET_FLAGS (button)  |= (GTK_CAN_DEFAULT));
  gtk_widget_grab_default (button); 
  gtk_widget_show (button);

  gtk_widget_show (about_dialog);

  return;
}

/*  -------------------------------------------------------------------- */
void view_manual()
{
  char    mpa[MAXPATH];
  char    sys[MAXPATH+20];

  g_snprintf (mpa, MAXPATH, "%s/manual.ps", MANUALDIR);
  if (0 != access(mpa, R_OK))
    {
	g_snprintf (mpa, MAXPATH, "%s", "./manual.ps");
        if (0 != access(mpa, R_OK))
          {
           g_message("Cannot find manual.");
           return;
          }
    }
#ifdef HAVE_GHOSTVIEW
        g_snprintf (sys, MAXPATH, "%s  %s &", PSVIEWER, mpa);
#else
        g_snprintf (sys, MAXPATH, "xv  %s &", mpa);
#endif
        system(sys);

return;
}

/*  -------------------------------------------------------------------- */
void 
enter_lambda (GtkWidget * entry, gpointer * text)
{
  gchar *entry_text;
  int num = atoi ((char *) text);
  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  lambda[num] = atof (entry_text);
  return;
}

/*  -------------------------------------------------------------------- */
void 
enter_of (GtkWidget * widget, gpointer * text)
{
  int num = atoi ((char *) text);

  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      iformat = (num     < 0) ? 0 : num;
      iformat = (iformat > 3) ? 3 : iformat;
    }

  return;
}

/*  -------------------------------------------------------------------- */

void 
enter_sigma (GtkWidget * entry, gpointer * text)
{
  gchar *entry_text;
  int num = atoi ((char *) text);
  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  sigma[num] = atof (entry_text);
  return;
}

/*  -------------------------------------------------------------------- */

void
enter_f(GtkWidget * entry, gpointer * text)
{
  gchar *entry_text;
  int num = atoi ((char *) text);
  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  f[num] = atof (entry_text);
  return;
}

/*  -------------------------------------------------------------------- */

void
enter_beta(GtkWidget * entry, gpointer * text)
{
  gchar *entry_text;
  int num = atoi ((char *) text);
  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  beta[num] = atof (entry_text);
  return;
}

/*  -------------------------------------------------------------------- */

void
enter_v(GtkWidget * entry, gpointer * text)
{
  gchar *entry_text;
  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  v = atof (entry_text);
  return;
}

/*  -------------------------------------------------------------------- */

void 
enter_any (GtkWidget * entry, char *text)
{
  gchar *entry_text;
  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  text = entry_text;
  return;
}

/*  -------------------------------------------------------------------- */
void
prof_use (GtkWidget * widget, gpointer * text)
{
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      iprof=1;
    }
  else
    {
      iprof=0;
    }
  return;
}
 
/*  -------------------------------------------------------------------- */
void 
eflag_tc (GtkWidget * widget, gpointer * text)
{
  int num = atoi ((char *) text);
  
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      pro_type[num] = P_LORENZ;
    }
  else
    {
      pro_type[num] = P_GAUSS;
    }
  return;
}

/*  -------------------------------------------------------------------- */
void 
eflag_wc (GtkWidget * widget, gpointer * text)
{
  int num = atoi ((char *) text);
  
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      fit_wave[num] = TRUE;
    }
  else
    {
      fit_wave[num] = FALSE;
    }
  return;
}

/*  -------------------------------------------------------------------- */
void 
eflag_sc (GtkWidget * widget, gpointer * text)
{
  int num = atoi ((char *) text);
  
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      fit_sigm[num] = TRUE;
    }
  else
    {
      fit_sigm[num] = FALSE;
    }
  return;
}

/*  -------------------------------------------------------------------- */
void 
eflag_lc (GtkWidget * widget, gpointer * text)
{
  int num = atoi ((char *) text);

  if (num > 0) 
    {
  
      if (GTK_TOGGLE_BUTTON (widget)->active)
	{
	  fit_lock[num] = TRUE;
	}
      else
	{
	  fit_lock[num] = FALSE;
	}
    }
  else  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (widget), FALSE);
      g_message("Cannot lock lambda[0] to previous line\n");
    }
  return;
}

/*  -------------------------------------------------------------------- */
/* Popup Window for nline wavelengths, sigmas and weight factors f       */

void 
pop_lines ()
{
  GtkWidget *buttonok;
  GtkWidget *box_command, *vbox, *box_loop;
  GtkWidget *label, *box_lab = NULL;
  GtkTooltips *ttip;

  int i = 0;
  char deftext[MAXTEXTL], labtext[MAXTEXTL];
  static char index[MAXLINES][3];
  static char i_dummy;
  static int withprof, init;

  if (init==0)
   {
    withprof=iprof;
    init=1;
   }

  ttip = gtk_tooltips_new();

  if (nlines < 1)
   {
     g_warning("nlines not positive, (nlines = %d)",nlines);
     return;
   }
  if (lines_open == 1)
    delete_lines (line_window, NULL);

  line_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (line_window), "Start values");
  gtk_signal_connect (GTK_OBJECT (line_window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_lines), NULL);

  /* For some reason I don't understand, the window does not expand
   * to the size of the widgets therein, but remains rather tiny.
   * Have to set a size manually therefore. This is an ugly kludge.
   */
  gtk_widget_set_usize(GTK_WIDGET(line_window), 600, nlines*50+100);  


  box_command = gtk_hbox_new (FALSE, 0);
  vbox = gtk_vbox_new (FALSE, 0);
  buttonok = gtk_button_new_with_label ("Ok");
  gtk_signal_connect_object (GTK_OBJECT (buttonok), "clicked",
			     GTK_SIGNAL_FUNC (delete_lines),
			     GTK_OBJECT (line_window));
  gtk_box_pack_start (GTK_BOX (box_command), buttonok, TRUE, TRUE, 0);
  gtk_widget_show (buttonok);
  gtk_widget_show (box_command);

  /*  put label atop 
   */
  box_lab = gtk_hbox_new (TRUE, 0);

  sprintf (labtext, "%s", " lambda");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_widget_show (label);

  sprintf (labtext, "%s", " sigma");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_widget_show (label);

  sprintf (labtext, "%s", " weight");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_widget_show (label);

  /* FLAGS
   */
  sprintf (labtext, "%s", " Profile");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_widget_show (label);

  sprintf (labtext, "%s", " Lambda");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_widget_show (label);

  sprintf (labtext, "%s", " Sigma");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_widget_show (label);

  sprintf (labtext, "%s", " LOCK");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_widget_show (label);

  sprintf (labtext, "%s", " Beta");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_lab), label, TRUE, TRUE, 2);
  gtk_box_pack_start (GTK_BOX (vbox), box_lab, TRUE, TRUE, 2);
  gtk_widget_show (label);
  gtk_widget_show (box_lab);

  for (i = 0; i < nlines; ++i)
    {
      /* The line number (argument to callback function)
       */
      sprintf (index[i], "%d", i);

      box_loop = gtk_hbox_new (TRUE, 0);

      /* -- lambda -- 
       */
      sprintf (deftext, "%f", lambda[i]);
      entry_line[i] = gtk_entry_new_with_max_length (15);
      if (i==0) sprintf (labtext, "%s %dst line", "central wavelength of",i+1);
      if (i==1) sprintf (labtext, "%s %dnd line", "central wavelength of",i+1);
      if (i==2) sprintf (labtext, "%s %drd line", "central wavelength of",i+1);
      if (i>2) sprintf (labtext, "%s %dth line", "central wavelength of",i+1);
   gtk_tooltips_set_tip(ttip,entry_line[i],labtext, NULL);

      gtk_signal_connect (GTK_OBJECT (entry_line[i]), "changed",
			  GTK_SIGNAL_FUNC (enter_lambda), &index[i][0]);
      gtk_entry_set_text (GTK_ENTRY (entry_line[i]), deftext);
      gtk_entry_select_region (GTK_ENTRY (entry_line[i]),
			       0, GTK_ENTRY (entry_line[i])->text_length);
      gtk_box_pack_start (GTK_BOX (box_loop), entry_line[i], TRUE, TRUE, 0);
      gtk_widget_show (entry_line[i]);

      /* -- sigma -- 
       */
      sprintf (deftext, "%7.6g", sigma[i]);
      entry_sig[i] = gtk_entry_new_with_max_length (15);
      if (i==0) sprintf (labtext, "%s %dst line", "line width of",i+1);
      if (i==1) sprintf (labtext, "%s %dnd line", "line width of",i+1);
      if (i==2) sprintf (labtext, "%s %drd line", "line width of",i+1);
      if (i>2) sprintf (labtext, "%s %dth line", "line width of",i+1);
   gtk_tooltips_set_tip(ttip,entry_sig[i],labtext, NULL);

      gtk_signal_connect (GTK_OBJECT (entry_sig[i]), "changed",
			  GTK_SIGNAL_FUNC (enter_sigma), &index[i][0]);
      gtk_entry_set_text (GTK_ENTRY (entry_sig[i]), deftext);
      gtk_entry_select_region (GTK_ENTRY (entry_sig[i]),
			       0, GTK_ENTRY (entry_sig[i])->text_length);
      gtk_box_pack_start (GTK_BOX (box_loop), entry_sig[i], TRUE, TRUE, 0);
      gtk_widget_show (entry_sig[i]);

      /* -- f (weight) --
       */
      sprintf (deftext, "%4.2f", f[i]);
      entry_f[i] = gtk_entry_new_with_max_length (6);
      if (i==0) sprintf (labtext, "%s%d for %dst line", "weighting factor f_",i+1,i+1);
      if (i==1) sprintf (labtext, "%s%d for %dnd line", "weighting factor f_",i+1,i+1);
      if (i==2) sprintf (labtext, "%s%d for %drd line", "weighting factor f_",i+1,i+1);
      if (i>2) sprintf (labtext, "%s%d for %dth line", "weighting factor f_",i+1,i+1);
   gtk_tooltips_set_tip(ttip,entry_f[i],labtext, NULL);

      gtk_signal_connect (GTK_OBJECT (entry_f[i]), "changed",
                          GTK_SIGNAL_FUNC (enter_f), &index[i][0]);
      gtk_entry_set_text (GTK_ENTRY (entry_f[i]), deftext);
      gtk_entry_select_region (GTK_ENTRY (entry_f[i]),
                               0, GTK_ENTRY (entry_f[i])->text_length);
      gtk_box_pack_start (GTK_BOX (box_loop), entry_f[i], TRUE, TRUE, 0);
      gtk_widget_show (entry_f[i]);

      sprintf (deftext, "%1d", pro_type[i]);
      eflag_t[i] = create_toggle (box_loop, eflag_tc, &index[i][0], 
				  deftext, "Lorentz");
      if (i==0) sprintf (labtext, "%s %dst line", "Use Lorentz Profile for",i+1);
      if (i==1) sprintf (labtext, "%s %dnd line", "Use Lorentz Profile for",i+1);
      if (i==2) sprintf (labtext, "%s %drd line", "Use Lorentz Profile for",i+1);
      if (i>2) sprintf (labtext, "%s %dth line", "Use Lorentz Profile for",i+1);
   gtk_tooltips_set_tip(ttip,eflag_t[i],labtext, NULL);

      sprintf (deftext, "%1d", fit_wave[i]);
      eflag_w[i] = create_toggle (box_loop, eflag_wc, &index[i][0], 
				  deftext, "Fit");
      if (i==0) sprintf (labtext, "%s %dst line", "Fitting of wavelength for",i+1);
      if (i==1) sprintf (labtext, "%s %dnd line", "Fitting of wavelength for",i+1);
      if (i==2) sprintf (labtext, "%s %drd line", "Fitting of wavelength for",i+1);
      if (i>2) sprintf (labtext, "%s %dth line", "Fitting of wavelength for",i+1);
   gtk_tooltips_set_tip(ttip,eflag_w[i],labtext, NULL);

      sprintf (deftext, "%1d", fit_sigm[i]);
      eflag_s[i] = create_toggle (box_loop, eflag_sc, &index[i][0], 
				  deftext, "Fit");
      if (i==0) sprintf (labtext, "%s %dst line", "Fitting of linewidth for",i+1);
      if (i==1) sprintf (labtext, "%s %dnd line", "Fitting of linewidth for",i+1);
      if (i==2) sprintf (labtext, "%s %drd line", "Fitting of linewidth for",i+1);
      if (i>2) sprintf (labtext, "%s %dth line", "Fitting of linewidth for",i+1);
   gtk_tooltips_set_tip(ttip,eflag_s[i],labtext, NULL);

      sprintf (deftext, "%1d", fit_lock[i]);
      eflag_l[i] = create_toggle (box_loop, eflag_lc, &index[i][0], 
				  deftext, "Lock");
      if (i==0) sprintf (labtext, "%s", "Inactive button!");
      if (i>0) sprintf (labtext, "%s %d", "Lock wavelength to line",i);
   gtk_tooltips_set_tip(ttip,eflag_l[i],labtext, NULL);


      /* -- beta
       */
      sprintf (deftext, "%4.2f", beta[i]);
      entry_beta[i] = gtk_entry_new_with_max_length (6);
      if (i==0) sprintf (labtext, "%s %dst line", "beta of",i+1);
      if (i==1) sprintf (labtext, "%s %dnd line", "beta of",i+1);
      if (i==2) sprintf (labtext, "%s %drd line", "beta of",i+1);
      if (i>2) sprintf (labtext, "%s %dth line", "beta of",i+1);
   gtk_tooltips_set_tip(ttip,entry_beta[i],labtext, NULL);

      gtk_signal_connect (GTK_OBJECT (entry_beta[i]), "changed",
                          GTK_SIGNAL_FUNC (enter_beta), &index[i][0]);
      gtk_entry_set_text (GTK_ENTRY (entry_beta[i]), deftext);
      gtk_entry_select_region (GTK_ENTRY (entry_beta[i]),
                               0, GTK_ENTRY (entry_beta[i])->text_length);
      gtk_box_pack_start (GTK_BOX (box_loop), entry_beta[i], TRUE, TRUE, 0);
      gtk_widget_show (entry_beta[i]);

      gtk_widget_show (box_loop);
      gtk_box_pack_start (GTK_BOX (vbox), box_loop, TRUE, TRUE, 0);

    }
sprintf (labtext, "use profile from %s",proffile);
if (withprof != 0)
 {
  profuse = create_toggle (vbox, prof_use, "1", deftext, labtext);
  if (iprof ==1) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (profuse), TRUE);         
 }

  /* v
   */
  box_loop = gtk_hbox_new (FALSE, 0);

  box_lab = gtk_hbox_new (FALSE, 0);
  sprintf (labtext, "%s", "");
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_loop), label, TRUE, FALSE, 2);
  gtk_widget_show (label);

  box_lab = gtk_hbox_new (FALSE, 0);
  label = gtk_label_new (labtext);
  gtk_box_pack_start (GTK_BOX (box_loop), label, TRUE, FALSE, 2);
  gtk_widget_show (label);

  box_lab = gtk_hbox_new (FALSE, 0);
  sprintf (labtext, "%s", " v=");
  label = gtk_label_new("  ----- Nlines\n"
			"  \\                       !\n" \
			"   >      f_i * A_i  = \n" \
			"  /\n" \
			"  ----- i=1\n");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start (GTK_BOX (box_loop), label, TRUE, FALSE, 2);
  gtk_widget_show (label);

  sprintf (deftext, "%f", v);
  entry_v = gtk_entry_new_with_max_length (15);
   gtk_tooltips_set_tip(ttip,entry_v,"Right side of equation for linear boundary condition", NULL);
  
  gtk_signal_connect (GTK_OBJECT (entry_v), "changed",
		      GTK_SIGNAL_FUNC (enter_v), &i_dummy);
  gtk_entry_set_text (GTK_ENTRY (entry_v), deftext);
  gtk_entry_select_region (GTK_ENTRY (entry_v),
			   0, GTK_ENTRY (entry_v)->text_length);
  gtk_box_pack_start (GTK_BOX (box_loop), entry_v, TRUE, TRUE, 0);
  gtk_widget_show (entry_v);
  gtk_widget_show (box_loop);
  gtk_box_pack_start (GTK_BOX (vbox), box_loop, TRUE, TRUE, 0);
  
  gtk_box_pack_start (GTK_BOX (vbox), box_command, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (line_window), vbox);
  gtk_widget_show (line_window);
  gtk_widget_show (vbox);

  lines_open = 1;

  return;
}

/*  -------------------------------------------------------------------- */

void 
update ()
{

  int i = 0;
  gchar ltext[MAXTEXTL];

  for (i = 0; i < NENTRY; ++i)
    {
      if (i == 0)
        sprintf (ltext, "%s", VERSION);
      if (i == 1)
        sprintf (ltext, "%s", fittitle);
      if (i == 2)
	sprintf (ltext, "%d", nlines);
      if (i == 3)
	sprintf (ltext, "%f", app_s);
      if (i == 4)
	sprintf (ltext, "%f", app_b);
      if (i == 32)
	sprintf (ltext, "%d", fixflag);
      if (i == 5)
	sprintf (ltext, "%f", sbg);
      if (i == 6)
	sprintf (ltext, "%d", sbgit);
      if (i == 23)
	sprintf (ltext, "%d", bgsmooth);
      if (i == 30)
	sprintf (ltext, "%d", sbgchi);
      if (i == 33)
	sprintf (ltext, "%5.3f", medsig);
      if (i == 7)
	sprintf (ltext, "%f", dsig);
      if (i == 8)
        sprintf (ltext, "%d", sigunit);
      if (i == 29)
	sprintf (ltext, "%d", ikev);
      if (i == 9)
	sprintf (ltext, "%f", dlam);
      if (i == 10)
	sprintf (ltext, "%d", ilamit);
      if (i == 11)
	sprintf (ltext, "%d", isigit);
      if (i == 13)
	sprintf (ltext, "%d", ires);
      if (i == 14)
	sprintf (ltext, "%d", iplot);
      if (i == 15)
	sprintf (ltext, "%d", ilab);
      if (i == 16)
	sprintf (ltext, "%d", ierrplot);
      if (i == 17)
	sprintf (ltext, "%d", ilog);
      if (i == 20)
	sprintf (ltext, "%d", ispline);
      if (i == 21)
	sprintf (ltext, "%d", idlam);
      if (i == 31)
        sprintf (ltext, "%d", idsig);
      if (i == 24)
	sprintf (ltext, "%d", iportrait);
      if (i == 26)
	sprintf (ltext, "%d", spec_only);
      if (i == 27)
	sprintf (ltext, "%d", fit_only);

      if (i == 1 || i == 2 || i == 3 || i == 4 || i == 5 || i == 7 || 
	  i == 9)
	{
	  gtk_entry_set_text (GTK_ENTRY (entry_arr[i]), ltext);
	}
      else
	{
	  if (i == 18 || i == 22 || i == 33)
	    {
	      if (i == 18)
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_arr[0]), 
					   nbin);
	      if (i == 22)
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_arr[1]), 
					   iord);
	      if (i == 33)
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_arr[2]),
					   medsig);
	    }
	  else if (i != 12 && i != 0 && i != 19 && i != 20)
	    {
	      if (atoi (ltext) == 1)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle[i]), 
					      TRUE);
	      else
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle[i]), 
					      FALSE);
	    }
	}
    }
  

  /*   updata line data 
   */
  if (lines_open == 0)
    pop_lines ();
  
  for (i = 0; i < nlines; ++i)
    {
      sprintf (ltext, "%f", lambda[i]);
      gtk_entry_set_text (GTK_ENTRY (entry_line[i]), ltext);
      sprintf (ltext, "%7.6g", sigma[i]);
      gtk_entry_set_text (GTK_ENTRY (entry_sig[i]), ltext);
      sprintf (ltext, "%f", f[i]);
      gtk_entry_set_text (GTK_ENTRY (entry_f[i]), ltext);

      if (pro_type[i] == P_LORENZ)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_t[i]), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_t[i]), FALSE);
      if (fit_wave[i] == TRUE)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_w[i]), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_w[i]), FALSE);
      if (fit_sigm[i] == TRUE)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_s[i]), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_s[i]), FALSE);
      if (fit_lock[i] == TRUE)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_l[i]), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (eflag_l[i]), FALSE);
    }
  sprintf (ltext, "%f", v);
  gtk_entry_set_text (GTK_ENTRY (entry_v), ltext);

  return;
}

/*  -------------------------------------------------------------------- */

void 
enter_spin (GtkWidget * spin, char *text)
{
  gchar buf[MAXTEXTL];

  int num = atoi ((char *) text);

  if (num == 22 || num == 33)
    sprintf (buf, "%f",
	gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (spin)));
  else
    sprintf (buf, "%d", 
	     gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)));
  
  if (num == 12)
    iformat = atoi (buf);
  if (num == 18)
    nbin = atoi (buf);
  if (num == 19)
    nlik = atoi (buf);
  if (num == 22)
    iord = atof (buf);
  if (num==25)
    verbose = atoi(buf);
  if (num==33)
    medsig = atof(buf);

}
/*  -------------------------------------------------------------------- */
void 
enter_switch (GtkWidget * widget, gpointer * text)
{
  int i;
  int num = atoi ((char *) text);
  
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      if (num == 8 && sigunit == 0)
	{
	  sigunit = 1;
	  dsig = dsig / lambda[0] * C_LIGHT;
	  for (i = 0; i < nlines; i = i + 1)
	    sigma[i] = sigma[i] / lambda[i] * C_LIGHT;
	  update ();
	}
      if (num == 6)
	sbgit = 1;
      if (num == 23)
	bgsmooth = 1;
      if (num == 30)
	sbgchi = 1;
      if (num == 10)
	ilamit = 1;
      if (num == 11)
	isigit = 1;
      if (num == 13)
	ires = 1;
      if (num == 14)
	iplot = 1;
      if (num == 15)
	ilab = 1;
      if (num == 16)
	ierrplot = 1;
      if (num == 17)
	ilog = 1;
      if (num == 20)
	ispline = 1;
      if (num == 21)
	idlam = 1;
      if (num == 31)
	idsig = 1;
      if (num == 24)
	iportrait = 1;
      if (num == 26)
	spec_only = 1;
      if (num == 27)
	fit_only = 1;
      if (num == 29)
	ikev = 1;
      if (num == 101)
	iuse_fixpt = 1;
      if (num == 32)
	fixflag = 1;
    }
  else
    {
      if (num == 8 && sigunit == 1)
	{
	  sigunit = 0;
	  dsig = lambda[0] * dsig / C_LIGHT;
	  for (i = 0; i < nlines; i = i + 1)
	    sigma[i] = lambda[i] * sigma[i] / C_LIGHT;
	  update ();
	}
      if (num == 6)
	sbgit = 0;
      if (num == 23)
	bgsmooth = 0;
      if (num == 30)
	sbgchi = 0;
      if (num == 10)
	ilamit = 0;
      if (num == 11)
	isigit = 0;
      if (num == 13)
	ires = 0;
      if (num == 14)
	iplot = 0;
      if (num == 15)
	ilab = 0;
      if (num == 16)
	ierrplot = 0;
      if (num == 17)
	ilog = 0;
      if (num == 20)
	ispline = 0;
      if (num == 21)
	idlam = 0;
      if (num == 31)
	idsig = 0;
      if (num == 24)
	iportrait = 0;
      if (num == 26)
	spec_only = 0;
      if (num == 27)
	fit_only = 0;
      if (num == 29)
	ikev = 0;
      if (num == 101)
	iuse_fixpt = 0;
      if (num == 32)
	fixflag = 0;
    }
  return;
}

/*  -------------------------------------------------------------------- */
/*  Put the entry into corresponding global variables  */
void 
enter_callback (GtkWidget * entry, gpointer * text)
{
  /*  
      static int test = 0;
      FILE *fd; 
  */
  gchar *entry_text;

  int num = atoi ((char *) text), itest=0;

  /* if (entry_text != NULL) free(entry_text); */

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  if (num == 1)
    sprintf (fittitle, "%s", entry_text);

  if (num == 2)
   {
    itest = atoi (entry_text);
    if (itest>MAXLINES)
	g_warning("%d no valid number of lines: [1...%d]",itest,MAXLINES);
    else if (itest > 0)
	nlines = itest;
   }
  if (num == 3)
    app_s = atof (entry_text);
  if (num == 4)
    app_b = atof (entry_text);
  if (num == 5)
    sbg = atof (entry_text);
  if (num == 7)
    dsig = atof (entry_text);
  if (num == 8)
    sigunit = atoi (entry_text);
  if (num == 29)
    ikev = atoi (entry_text);
  if (num == 9)
    dlam = atof (entry_text);

  /* Popup Window for nline wavelengths and sigmas 
   */
  if (num == 2 && itest>0 && itest <= MAXLINES)
    pop_lines ();

  if (num == 28) rem_index = atoi (entry_text);

  return;
}

/*  -------------------------------------------------------------------- */

void
remove_line (int ciflag)
{

int i,j=1;

if (rem_index>nlines || rem_index <= 0)
 {
  g_warning("Cannot remove line number %d (Maximum index %d)\n",rem_index,nlines);
  return;
 }

--nlines;

while (j<rem_index) ++j;
if (verbose)
   g_message ("Removing line %d of %d",j,nlines+1);
for (i=j-1;i<nlines;++i)
  {
    lambda[i]=lambda[i+1];
    sigma[i]=sigma[i+1];
    f[i]=f[i+1];
    pro_type[i]=pro_type[i+1];
    fit_wave[i]=fit_wave[i+1];
    fit_sigm[i]=fit_sigm[i+1];
    fit_lock[i]=fit_lock[i+1];
    beta[i]=beta[i+1];
  }

if (ciflag == 1) update();

return;
}

void
remove_line_u ()
{
remove_line(1);
return;
}

void
ask_remove ()
{

  GtkWidget   *reml_win;
  GtkWidget   *button;
  GtkWidget   *entry;
  GtkWidget   *box1;

  reml_win = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_position(GTK_WINDOW(reml_win), GTK_WIN_POS_MOUSE);
  gtk_signal_connect (GTK_OBJECT (reml_win), "destroy",
                      GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
  box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (reml_win), box1);
  gtk_container_border_width (GTK_CONTAINER (box1), 5);

  entry=create_entry(box1, enter_callback, "28", "1", "Enter line index of line to be removed");

  button = gtk_button_new_with_label ("Confirm");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (remove_line_u), NULL);
  gtk_box_pack_start (GTK_BOX (box1), button, TRUE, TRUE, 0);
  (GTK_WIDGET_FLAGS (button)  |= (GTK_CAN_DEFAULT));
  gtk_widget_grab_default (button); 
  gtk_widget_show (button);

  button=gtk_button_new_with_label("Close");
  gtk_container_add(GTK_CONTAINER(box1), button);
  gtk_widget_show(button);
  gtk_signal_connect_object(GTK_OBJECT(button),
                              "clicked",
                              GTK_SIGNAL_FUNC(gtk_widget_destroy),
                              GTK_OBJECT(reml_win));

  gtk_widget_show (reml_win);

return;
}

/*  -------------------------------------------------------------------- */
/*  Put the global variables into input file  */
void
make_inputfile_int (char * path)
{
  FILE *fd;
  int i = 0;

  if (path == NULL)
    return;

  if (nlines < 1 )
    {
      g_warning("input file cannot be created when no line is specified");
      return;
    }

  if (verbose) g_message ("saving current settings to %s", path);

  fd = fopen (path, "w");
  if (!fd)
    {
      i = errno;
      g_warning ("fopen: %s: %s", path, g_strerror (i));
      return;
    }

  fprintf (fd, "%s              ; %s\n", VERSION,"Version");
  if (fittitle[0] == '\0') fprintf (fd, "%s \n", "NULL");
   else fprintf (fd, "%s \n", fittitle);
  fprintf (fd, "%d              ; %s\n", nlines,"number of lines");
  for (i = 0; i <= nlines - 1; ++i)
    fprintf (fd, "%5.4f,%6.4f,%4.3f,%d,%d,%d,%d,%4.3f ; %s_%d, %s_%d, %s_%d %s_%d\n",
             lambda[i], sigma[i], f[i],
             pro_type[i], fit_wave[i], fit_sigm[i], fit_lock[i],beta[i],
             "lam",i,"sig",i,"f",i,
             "Flags: Prof, FitLam, FitSig, LockLam, Bet",i);
  fprintf (fd, "%4.3f,%4.3f,%6.5f,%d         ; %s\n", app_s, app_b,v,fixflag,
           "App(source),App(BG),v,fixflag");
  fprintf (fd, "%6.4f,%d,%d,%d,%5.4f   ; %s\n", sbg, sbgit, bgsmooth,sbgchi,medsig,
           "source background [cts/bin],=1:iterate sbg,=1 smooth background,fit sbg with chi,median sbg range");
  fprintf (fd, "%5.4f,%d,%5.4f,%d       ; %s\n", dsig, sigunit, dlam, ikev,
           "dsig, unit(sig), dlam, ikev");
  fprintf (fd, "%d,%d,%5.3f,%d   ; %s\n", ilamit,idlam,iord,idsig,
           "ilamit, idlam, iord, idsig");
  fprintf (fd, "%d              ; %s\n", isigit,
           "isigit(=1 automatic iteration in sigma)");
  fprintf (fd, "%d              ; %s\n", ires,
           "ires=1 write results into file");
  fprintf (fd, "%d,%d            ; %s\n", iformat,iportrait,
           "iformat: 0: X-window, 1: PS, 2:GIF, 3: PPM, "\
           "iportrait for orientation");
  fprintf (fd, "%d,%d,%d,%d,%d,%d,%d    ; %s\n",
           iplot, ilab, ierrplot, ilog, nbin,spec_only,fit_only,
           "iplot,ilab,ierrplot,ilog,nbin,spec_only,fit_only");
  fprintf (fd, "%d,%d            ; %s\n", nlik, ispline,
           "nlik, ispline (not supported by C program)");

  /* nlik (>0 plot liklihood curves with nlik points),ispline"); */

  fclose  (fd);
  return;
}

void make_inputfile ()
{
  char path[256];
  int i=0,rflag=0;

  if (verbose) g_message ("removing lines with lambda=0");
  while (i<nlines)
   {
    if (lambda[i]==0.) {rem_index=i+1;remove_line(0);--i;rflag=1;}
    ++i;
   }

  if (rflag==1) update();

  make_inputfile_int (conffile);

  g_snprintf(path, 255, "%s/.cora/fit.inp", g_get_home_dir());

  if (strcmp(path, conffile) == 0)
    return;

  if (0 != test_confdir())
    return;

  make_inputfile_int (path);

  return;
}

/*  -------------------------------------------------------------------- */
static char * check_name(char * name, char * ending)
{
  int size;
  int siz2;
  char * p;
  char myname[] = "cora";

  if (name == NULL)
    name = myname;

  size = strlen(name);
  siz2 = strlen(ending);

  if (size > siz2 && 0 == strcmp (&name[size-siz2], ending))
    {
      p = malloc (size+1);
      strcpy(p, name);
    }
  else
    {
      p = malloc (size+siz2+1);
      strcpy(p, name);
      strcat(p, ending);
    }
  return p;
}

/*  ------------------------ */

#if defined(USE_TIMEOUT)
static int alarm_rung = 0;

static void sh_alarmhandle (int signal)
{
  struct  sigaction  def_act;
  def_act.sa_handler = SIG_DFL;

  alarm_rung = 1;
  alarm(0);
  sigaction (SIGALRM, &def_act, NULL);
  return;
}
#endif

/* Start cora_fit */
void 
Go (GtkWidget * widget, gpointer data)
{

  FILE  * fd;
  char    line[1024];
  char    command[256];
  char	  comm_cpy[256];
  char    sys[MAXPATH+20];
  char  * endptr;
  char    c_dummy[10];
  int     lines_recv = 0;
  double  valueL[MAXLINES];
  double  valueS[MAXLINES];
  double  valuesbg = sbg;
  int     cerr = 0;
  int     i;

#if defined(USE_TIMEOUT)
  int     i_alarm = 0;
  struct  sigaction  def_act;
  struct  sigaction  new_act;
  def_act.sa_handler = SIG_DFL;
  new_act.sa_handler = sh_alarmhandle;
  alarm_rung = 0;
#endif

  make_inputfile ();
  /*  
      if (!verbose && ires ==0 && iplot ==0)
      {
      g_message ("No output device selected, abort fit.");
      return;
      }
  */

  if (verbose) 
    g_message ("Calling fit routine ...");


  /*  
   * fd = popen ("echo '@do' | $IDL_DIR/bin/idl 2>&1", "r"); 
   */

  g_snprintf (c_dummy,10," ");
  if (verbose == 1) g_snprintf (c_dummy,10," -v");
  if (verbose == 2) g_snprintf (c_dummy,10," -v -v");
  if (verbose == 3) g_snprintf (c_dummy,10," -v -v -v");

  if (0 == access(conffile, R_OK) && 0 == access(datafile, R_OK))
      g_snprintf (command, 255, "cora_fit -s %s -i %s%s",datafile,conffile,c_dummy);
  else
    {
      g_warning ("%s or %s not accessible", datafile, conffile);
      return;
    } 

 g_snprintf (comm_cpy, 255,"%s",command);
 if (0 != strcmp (resfile,"cora.dat"))
  {
    g_snprintf (command, 255,"%s -o %s",comm_cpy,resfile);
    g_snprintf (comm_cpy, 255,"%s",command);
  }
 if (0 != strcmp (plotfile,"cora"))
  {
    g_snprintf (command, 255,"%s -p %s",comm_cpy,plotfile);
    g_snprintf (comm_cpy, 255,"%s",command);
  }
 if (ireplot==1)
  {
    g_snprintf (command, 255,"%s -r %s",comm_cpy,resfile);
    g_snprintf (comm_cpy, 255,"%s",command);
  }
 if (dt != 1.)
  {
    g_snprintf (command, 255,"%s -e %f",comm_cpy,dt);
    g_snprintf (comm_cpy, 255,"%s",command);
  }
 if (iprof == 1 && 0==access(proffile,R_OK))
  {
   g_snprintf (command, 255,"%s -l %s",comm_cpy,proffile);
   g_snprintf (comm_cpy, 255,"%s",command);
  }
  g_snprintf (command, 255,"%s 2>&1 &",comm_cpy);

  if (verbose)
    g_message ("executing %s", command);

  fd = popen (command, "r");
  if(!fd)
    {
      g_warning ("popen() failed !");
      return;
    }
  
  /* --- FIXME -- Redirect cora_fit output --- */
  
  while (NULL != fgets (line, 1023, fd))
    {
#if defined(USE_TIMEOUT)
      if (alarm_rung)
	{
	  if (verbose > 1)
	    {
	      g_message("fgets: timeout.");
	    }
	  break;
	}
      if (line[0] == ';' && i_alarm == 0)
	{
	  i_alarm = 1;
	  alarm(0);
	  sigaction (SIGALRM, &new_act, NULL);
	  alarm(3);
	}
#else
      if (line[0] == ';' && line[1] == ';')
	{
	  if (verbose > 1)
	    {
	      g_message("fgets: EOF sign (double ;;) received.");
	    }
	  break;
	}
#endif
      if (line[0] == '*')
	{
	  insert_in_buffer (line);
	  if (!quiet)
	    fprintf(stderr, "%s\n", line);
	}
      else
	{

	  insert_in_buffer (line);
	  if (!quiet)
	    fprintf(stdout, "%s\n", line);

	  if (line[2]=='s' && line[3]=='b' && line[4]=='g')
	    valuesbg = strtod(&line[11], &endptr);
	  else
	    {
	      if (line[0] != ';' || strlen (line) < 22)
		continue;
	      else if (lines_recv < MAXLINES)
		{
		  valueL[lines_recv] = strtod(&line[2], &endptr);
		  if (valueL[lines_recv] == 0 || endptr == &line[2])
		    continue;
		  valueS[lines_recv] = strtod(&line[23], &endptr);
		  if (valueS[lines_recv] == 0 || endptr == &line[23])
		    continue;
		  ++lines_recv;
		  /*
		    printf("recv %d %7.4f %7.4f\n", 
		    lines_recv, valueL[lines_recv-1], 
		    valueS[lines_recv-1]);
		  */
		}
	    }
	}  
    }
  
  cerr = pclose (fd);

#if defined(USE_TIMEOUT)
  sigaction (SIGALRM, &def_act, NULL);
#endif

  if (cerr == 0)
    {
      g_message ("Finished fit routine ... exit status %d", 
		 cerr);
      if (ireplot ==0 )
	{
	  for (i = 0; i < lines_recv; ++i)
	    {
	      lambda[i] = valueL[i]/iord;
	      sigma[i]  = valueS[i];
	    }
	  sbg = valuesbg;
	  /* nlines = lines_recv; */
	  
	  update ();
	}
      
      if(iplot == 1)
	{
	  if(iformat == 1)
#ifdef HAVE_GHOSTVIEW 
	    g_snprintf (sys, MAXPATH+20, "%s  %s &", PSVIEWER, 
			check_name(plotfile, ".ps"));
#else
	  g_snprintf (sys, MAXPATH+20, "xv  %s &", 
		      check_name(plotfile, ".ps"));
#endif                         
	  if(iformat == 2) 
	    g_snprintf (sys, MAXPATH+20, "xv %s &", 
			check_name(plotfile, ".gif"));
	  if(iformat == 3) 
	    g_snprintf (sys, MAXPATH+20, "xv %s &", 
			check_name(plotfile, ".ppm"));
	  if (iformat > 0) 
	    system (sys);
	}
      
    }
  else
      g_warning ("Error in fit routine : %s", g_strerror (cerr));

  return;
}

/*  -------------------------------------------------------------------- */

void
cora_tex(int ciflag)
 {
  char    command[256];
 
  if (0 == access(resfile, R_OK))
    {
      g_snprintf (command, 255, "cora_tex -i %s -t %s 2>/dev/null",
                  resfile, texfile);
      if (verbose > 1)
        g_message("executing %s",command);
      system (command);
    }
  else
    g_warning ("No read access for %s", resfile);
  return;
 }

/*  -------------------------------------------------------------------- */

void
cora_inp(int ciflag)
 {
  char    command[256];

  if (0 == access(datafile, R_OK))
    {
      g_snprintf (command, 255, "cora_inp -s %s -i %s 2>/dev/null", 
		  datafile, conffile);
      if (verbose > 1) 
	g_message("executing %s",command);
      system (command);
      read_input (conffile, 1);
      if (ciflag == 1) 
	update();
    }
  else
    g_warning ("No read access for %s", datafile);
  return;
 }

void
cora_inp_u()
{
/*  updates GUI after creating new input file  */
  cora_inp(1);
  return;
}

void
cora_inp_nu()
{
/*  no update of GUI after creating new input file  */
  cora_inp(0);
  return;
}

/*  -------------------------------------------------------------------- */

void 
file_ok_read (GtkWidget * w, GtkFileSelection * fs)
{
  char filename_sel[MAXPATH+1];
  
  g_snprintf (filename_sel, MAXPATH, "%s", 
	      gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  read_input (filename_sel, 1);
  gtk_widget_destroy (GTK_WIDGET (fs));
  update ();
  g_snprintf (conffile, MAXPATH, "%s", filename_sel);
  save_setting("parameters", conffile);

}

/*  -------------------------------------------------------------------- */
void
spec_ok_read (GtkWidget * w, GtkFileSelection * fs)
{
  char filename_sel[MAXPATH+1];
  float * s_wave;
  float * s_flux;
  float * s_err;
  float * s_bb;
  int     s_count = count;

  s_wave = (float *) malloc(sizeof(float) * s_count);
  s_flux = (float *) malloc(sizeof(float) * s_count);
  s_err  = (float *) malloc(sizeof(float) * s_count);
  s_bb   = (float *) malloc(sizeof(float) * s_count);
  
  memcpy (s_wave, wave, sizeof(float) * s_count);
  memcpy (s_flux, flux, sizeof(float) * s_count);
  memcpy (s_err,   err, sizeof(float) * s_count);
  memcpy (s_bb,     bb, sizeof(float) * s_count);

  g_snprintf (filename_sel, MAXPATH, "%s",
              gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  read_input (filename_sel, 1);
  gtk_widget_destroy (GTK_WIDGET (fs));

  free(wave); wave = NULL; 
  free(flux); flux = NULL;
  free(err);   err = NULL;
  free(bb);     bb = NULL; 
  count=0;

  if(0 >= read_spec(filename_sel, 0))
    {

      wave = (float *) malloc(sizeof(float) * s_count);
      flux = (float *) malloc(sizeof(float) * s_count);
      err  = (float *) malloc(sizeof(float) * s_count);
      bb   = (float *) malloc(sizeof(float) * s_count);

      g_warning ("No new spectrum data read in from %s.", filename_sel);

      memcpy (wave, s_wave, sizeof(float) * s_count);
      memcpy (flux, s_flux, sizeof(float) * s_count);
      memcpy (err,   s_err, sizeof(float) * s_count);
      memcpy (bb,     s_bb, sizeof(float) * s_count);
      count = s_count;
    }
  else
    {
      g_snprintf (datafile, MAXPATH, "%s", filename_sel);
      popup_message("A new parameter file adapted for the new spectrum can be\ncreated with Options/Create Standard Inputfile");
      g_message ("A new parameter file adapted for the new spectrum can be");
      g_message ("created with Options/Create Standard Inputfile\n");
      save_setting("datafile", datafile);
    }

  free (s_wave); free (s_flux); free (s_err); free (s_bb);
  return;

}

/*  -------------------------------------------------------------------- */

void
file_ok_save (GtkWidget * w, GtkFileSelection * fs)
{
  char filename_sel[MAXPATH+1];
  
  g_snprintf (filename_sel, MAXPATH, "%s",
              gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
  g_snprintf (conffile, MAXPATH, "%s", filename_sel);
  gtk_widget_destroy (GTK_WIDGET (fs));
  make_inputfile();
  update ();
  save_setting("parameters", conffile);
  return;
}

/*  -------------------------------------------------------------------- */

void 
graph ()
{
  float xpos[MAXLINES];
  float ypos = 0.0, b_test=0.0;
  int i = 0, graph_lines;

  if (0 == getlambda (xpos, &ypos, &graph_lines))
   {
    if (verbose>1)
      g_message ("xpos(0)=%f ;  %d lines selected\n", xpos[0], graph_lines);
   }
  else
    {
      g_warning ("Datafile %s missing.", datafile);
      return;
    }

  /* These are the lines selected by the user. Line number is graph_lines,
   * line positions are in xpos.
   */
  if(graph_lines > 0)
   {
    nlines = graph_lines;
    for (i = 0; i < MAXLINES; ++i)
      {
        if (i < nlines)
	  lambda[i] = xpos[i];
        else
	  lambda[i] = 0.0;
      }
   }

  /* This (ypos) is the background level selected by the user.
   */
  if (ypos != 2.*(wave[count-1]-wave[0])/count)
   {
     /* This is a really cryptic routine that should be commented
      * by the author. 
      */
     i=0;
     
     while(fabs(wave[i]-lambda[0]) > 0.1)
       {
	 b_test = (bb[i] + bb[i+1] + bb[i+3]) / 3.;
	 ++i;
       }
     if (ypos >= b_test)
       {
	 sbg = ypos-b_test;
	 sbg = sbg/(wave[count-1]-wave[0])*count;
       }
   }

  update ();
}

/*  -------------------------------------------------------------------- */

void 
file_sel (GtkWidget * w, GtkFileSelection * fs)
{
  GtkWidget *filew;
  char      * deff;

  /* Create a new file selection widget */
  filew = gtk_file_selection_new ("Select parameter file");
  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, &filew);

  /* Connect the ok_button to file_ok_read function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", (GtkSignalFunc) file_ok_read, filew);

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT 
			     (GTK_FILE_SELECTION (filew)->cancel_button), 
			     "clicked", (GtkSignalFunc) gtk_widget_destroy, 
			     GTK_OBJECT (filew));

  /* Lets set the filename, as if this were a save dialog, and we are giving
     a default filename */
  deff = get_setting ("parameters");
  if (deff == NULL)
    deff = conffile;
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew),
                                   deff);
  gtk_widget_show (filew);
}

/*  -------------------------------------------------------------------- */

void
spec_sel ()
{
  GtkWidget * filew;
  char      * deff;

  /* Create a new file selection widget */
  filew = gtk_file_selection_new ("Select ASCII file with spectrum");
  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
                      (GtkSignalFunc) gtk_widget_destroy, &filew);

  /* Connect the ok_button to file_ok_read function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
                      "clicked", (GtkSignalFunc) spec_ok_read, filew);

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT
                             (GTK_FILE_SELECTION (filew)->cancel_button),
                             "clicked", (GtkSignalFunc) gtk_widget_destroy,
                             GTK_OBJECT (filew));

  /* Lets set the filename, as if this were a save dialog, and we are giving
     a default filename */
  deff = get_setting ("datafile");
  if (deff == NULL)
    deff = datafile;
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew),
                                   deff);
  gtk_widget_show (filew);
}

/*  -------------------------------------------------------------------- */

void
save_as()
{
  GtkWidget *filew;

  /* Create a new file selection widget */
  filew = gtk_file_selection_new ("File selection");
  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
                      (GtkSignalFunc) gtk_widget_destroy, &filew);

  /* Connect the ok_button to file_ok_save function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
                      "clicked", (GtkSignalFunc) file_ok_save, filew);

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT
                             (GTK_FILE_SELECTION (filew)->cancel_button),
                             "clicked", (GtkSignalFunc) gtk_widget_destroy,
                             GTK_OBJECT (filew));

  /* Lets set the filename, as if this were a save dialog, and we are giving
     a default filename */
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew),
                                   conffile);

  gtk_widget_show (filew);
}

/*  -------------------------------------------------------------------- */

void
replot_f()
{
  FILE  * fd;
  char    line[1024];
  char    command[256];
  char    sys[MAXPATH+20];
  int     cerr = 0;

  make_inputfile ();

  if (0 == access(conffile, R_OK) && 0 == access(datafile, R_OK) && 
      0 == access(resfile,  R_OK))
    {
      if(!verbose) 
	g_snprintf (command, 255, "cora_fit -s %s -i %s -r %s -p %s -e %f 2>&1",
		    datafile,conffile,resfile,plotfile,dt);
      else
	{
	  if (verbose == 1) 
	    g_snprintf (command, 255, 
			"cora_fit -v -s %s -i %s -r %s -p %s -e %f 2>&1",
			datafile,conffile,resfile,plotfile,dt);
	  else 
	    g_snprintf (command, 255, 
			"cora_fit -v -v -s %s -i %s -r %s -p %s -e %f 2>&1",
			datafile,conffile,resfile,plotfile,dt);
	}
    }
  else
    {
      g_warning ("%s, %s or %s not accessable", datafile,conffile,resfile);
      return;
    } 
  
  if (verbose) 
    g_message ("executing %s",command);

  fd = popen (command, "r");
  if(!fd)
    {
      g_warning ("popen() failed !");
      return;
    }
  
  while (NULL != fgets (line, 1023, fd))
    {
      if (line[0] == ';' && line[1] == ';')
        {
          if (verbose > 1)
            {
              g_message("fgets: EOF sign (double ;;) received.");
            }
          break;
        }

      insert_in_buffer (line);
      if (!quiet)
	fprintf(stderr, "%s\n", line);
    }  
  cerr = pclose (fd);
  
  if (cerr == 0)
    {
      if (verbose) 
	g_message ("Finished program... exit status %d",
		   cerr);
      if(iplot == 1)
	{
	  if(iformat == 1) 
#ifdef HAVE_GHOSTVIEW
	    g_snprintf (sys, MAXPATH+20, "%s  %s &", PSVIEWER,
			check_name(plotfile, ".ps"));
#else
	  g_snprintf (sys, MAXPATH+20, "xv  %s &",
		      check_name(plotfile, ".ps"));
#endif
	  if(iformat == 2) 
	    g_snprintf (sys, MAXPATH+20, "xv %s &",
			check_name(plotfile, ".gif"));
	  if(iformat == 3) 
	    g_snprintf (sys, MAXPATH+20, "xv %s &",
			check_name(plotfile, ".ppm"));
	  if (iformat>0) 
	    system(sys);
	}
    }
  else
    {
      g_warning ("Error in replot routine : %s", g_strerror (cerr));
    }

  return;
}

/*  -------------------------------------------------------------------- */

void
replot()
{
  char test[MAXPATH];

  g_snprintf (test, MAXPATH, "%s", resfile);
  g_snprintf (resfile, MAXPATH, "%s/.cora/cora.dat", g_get_home_dir());

  replot_f();

  g_snprintf (resfile, MAXPATH, "%s", test);

}

/*  -------------------------------------------------------------------- */

void
set_sig_zero()
{

 int i=0;

 for (i=1;i<nlines;++i) sigma[i]=0.;
 update();

}

/*  -------------------------------------------------------------------- */

static GtkItemFactoryEntry menu_items[] =
{
  {"/_File", NULL, NULL, 0, "<Branch>"},
  {"/File/_New", "<control>N", cora_inp_u, 0, NULL},
  {"/File/Open _Parameter File", "<control>P", file_sel, 0, NULL},
  {"/File/Open _Data File", "<control>D", spec_sel, 0, NULL},
  {"/File/_Save", "<control>S", make_inputfile, 0, NULL},
  {"/File/Save _As", NULL, save_as, 0, NULL},
  {"/File/sep1", NULL, NULL, 0, "<Separator>"},
  {"/File/Quit", "<control>Q", delete_event, 0, NULL},
  {"/_Options", NULL, NULL, 0, "<Branch>"},
  {"/Options/_Graph", "<control>G", graph, 0, NULL},
  {"/Options/Go", NULL, Go, 0, NULL},
  {"/Options/Remove line", NULL, ask_remove, 0, NULL},
  {"/Options/Replot _Last", "<control>L", replot, 0, NULL},
  {"/Options/Replot From _File", "<control>F", replot_f, 0, NULL},
  {"/Options/Create Standard Inputfile",NULL, cora_inp_u, 0, NULL},
  {"/Options/Set Sigmas Zero",NULL, set_sig_zero, 0, NULL},
  {"/Options/Create TeX file from log-file",NULL, cora_tex, 0, NULL},
  {"/Options/ShowLog", NULL, log_event, 0, NULL},
  {"/_Help", NULL, NULL, 0, "<LastBranch>"},
  {"/_Help/About", NULL, about_cb, 0, NULL},
  {"/_Help/Manual",NULL, view_manual, 0, NULL}
};

/*  -------------------------------------------------------------------- */

void 
get_main_menu (GtkWidget * window, GtkWidget ** menubar)
{
  int nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
     or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
     the accelerator table while generating menus.
   */

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));

  if (menubar)
    /* Finally, return the actual menu bar created by the item factory. */
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");

  return;
}

/*  -------------------------------------------------------------------- */

/* MMMMMMMMMMMMMMMMMMMMMMMM Main MMMMMMMMMMMMMMMMMMMMMMMMM */
int 
main (int argc,
      char *argv[])
{
  /* GtkWidget is the storage type for widgets */
  GtkWidget *window;
  GtkWidget *buttongo, *buttonsave, *buttonquit;
  GtkWidget *box_command, *box_loop = NULL, *vbox, *hbox;
  GtkWidget *separator;

  GtkWidget *notebook;    
  GtkWidget *label;    
  GtkWidget *vbox0;    

  GtkAdjustment *adj;
  GtkWidget *menubar;
  GtkWidget *button, *box_tmp = NULL, *box_tmp1;
  GSList *group;

  GSList *groupx;
  GtkWidget *button1, *button2, *button3, *button4;

  GtkTooltips *ttip;

  int  i,fnotfound=0;
  char deftext[MAXTEXTL], labtext[MAXTEXTL], usage[MAXPATH];
  char ppath[MAXPATH],command[256];

  strncpy (datafile, "spectrum.dat", MAXPATH);
  strncpy (conffile, "fit.inp", MAXPATH);
  strncpy (proffile, "profile.dat",MAXPATH);
  strncpy (resfile, "cora.dat", MAXPATH);
  strncpy (texfile, "cora.tex", MAXPATH);
  strncpy (plotfile, "cora", MAXPATH);

  strncpy (usage, "Usage: cora [-help] [-s spectrum_file] [-i parameter_file] "\
	   "[(-o|-r) result_file] [-p plot_file] [-e expos. time] [-l lineprofile_file]\n"\
	   "            [-v] [-q]\n", 
	   MAXPATH);

  /* initialize to default: gauss profile, fit lambda
   */
  for(i=0; i < MAXLINES; ++i)
   {
    fit_wave[i] = TRUE;
    fit_sigm[i] = TRUE;
    fit_lock[i] = FALSE;
    pro_type[i] = P_GAUSS;
    beta[i] = 1.;
   }

  while (argc > 1 && argv[1][0] == '-')
    {
      switch (argv[1][1])
	{
	case 'V':
	  fprintf (stderr, "Cora line fitting tool Version %s\n",VERSION);
	  exit (8);
	case 'v':
          ++verbose;
          break;
        case 'q':
          quiet = 1;
          break;
	case 's':
	  ++argv;
	  --argc;
	  if (argc == 1)
	    {
	      fprintf (stderr, usage);
	      exit (8);
	    }
	  strncpy (datafile, argv[1], MAXPATH);
	  break;
        case 'l':
          ++argv;
          --argc;
          if (argc == 1)
            {
              fprintf (stderr, usage);
              exit (8);
            }
          strncpy (proffile, argv[1], MAXPATH);
	  iprof = 1;
          break;
	case 'i':
	  ++argv;
	  --argc;
	  if (argc == 1)
	    {
	      fprintf (stderr, usage);
	      exit (8);
	    }
	  strncpy (conffile, argv[1], MAXPATH);
	/*   create conffile if not present  */
	  if (0 != access(conffile, R_OK))
	      fnotfound=1;
	  break;
	case 'p':
	  ++argv;
	  --argc;
	  if (argc == 1)
	    {
		fprintf (stderr, usage);
		exit (8);
	    }
	  strncpy (plotfile, argv[1], MAXPATH);
	  break;
	case 'o':
	  ++argv;
	  --argc;
	  if (argc == 1)
	    {
		fprintf (stderr, usage);
		exit (8);
	    }
	  strncpy (resfile, argv[1], MAXPATH);
	  break;
	case 'r':
	  ++argv;
	  --argc;
	  if (argc == 1)
	    {
		fprintf (stderr,usage);
		exit (8);
	    }
	  strncpy (resfile, argv[1], MAXPATH);
	  ireplot=1;
	  break;
	case 'e':
	  ++argv;
	  --argc;
	  if (argc == 1)
	   {
	    fprintf (stderr, usage);
	    exit (8);
	   }
	  dt = atof(argv[1]);
	  printf("* Warning *: The exposure time will take effect in the log file, which should not be used for replots\n");
	  break;
	default:
	  fprintf (stderr, usage);
          fprintf (stderr, "%s","spectrum file    : spectrum with two, three, or five columns (create example with cora_spec)\n");
          fprintf (stderr, "%s","parameter file   : Input file (create default with cora_inp)\n");
          fprintf (stderr, "%s","result_file      : file containing fit results:\n");
          fprintf (stderr, "%s","  -o             : indicate file to be written into\n");
          fprintf (stderr, "%s","  -r             : indicate file to be read from (for replot)\n");
          fprintf (stderr, "%s","plot_file        : (ps, gif, or ppm) file\n");
          fprintf (stderr, "%s","expos. time      : exposure time (can also be given in first line of spectrum file)\n");
          fprintf (stderr, "%s","lineprofile_file : file with numeric profile function (two columns)\n");
          fprintf (stderr, "%s","-v               : verbosity (up to three times)\n");
          fprintf (stderr, "%s","-q               : Quiet mode\n");
	  exit (8);
	  break;
	}
      ++argv;
      --argc;
    }

  if(fnotfound == 1)
    {
	g_warning("Parameter file %s not found, creating it.", conffile);
	cora_inp_nu();
    }

  if (NULL == getenv("PGPLOT_DIR") && NULL == getenv("PGPLOT_FONT"))
    {
      fprintf(stderr, 
	      "** WARNING **: Neither the environment variable PGPLOT_DIR\n"\
	      "*              neither the environment variable PGPLOT_FONT\n"\
	      "*              are defined.\n"\
	      "*              This means that PGPLOT will not be able to\n"\
	      "*              locate the binary font file 'grfont.dat', and\n"\
	      "*              thus plot labels may be corrupted or missing.\n"\
	      "*              Please consult your system administrator for\n"\
	      "*              proper setting of (at least) one of these two\n"\
	      "*              environment variables.\n");
    }

  g_log_set_handler (NULL, 
		     G_LOG_LEVEL_MESSAGE |  
		     G_LOG_LEVEL_WARNING | 
		     G_LOG_FLAG_FATAL, 
		     my_log_handler, NULL);


  /*  read data file with spectrum or argv[]  
   */
  if((0 != access(datafile, R_OK)) || (0 >= read_spec(datafile, 0)))
    {
      g_warning("No spectrum data read in from %s.", datafile);
    }


  /*  define default wavelength and default sigma
    dummy     = float_max(flux,count,&index);
    lambda[0] = wave[index];
    while(flux[index] > dummy/2.) 
       --index;

    sigma[0] = lambda[0]-wave[index];
    dsig = sigma[0]*0.9;
    dlam = 15*dsig;
   */

  /*  Read parameter file.
   *  First search in $PWD.
   *  On failure, fallback to $HOME/.cora
   *  Last resort, create one in $PWD
   */
  i = 1;

  if (0 == access(conffile, R_OK))
    {
      if (verbose)
	g_message ("Using parameter file %s", 
		   conffile);
      g_snprintf(ppath, MAXPATH, "%s", conffile);
    }
  else if (NULL != get_setting("parameters"))
    {
      g_snprintf(ppath, MAXPATH, "%s", get_setting("parameters"));
      if (verbose)
	g_message ("Parameter file %s not found, try %s", 
		   conffile, ppath);
    }
  if (0 != access(ppath, R_OK))
    {
      if (verbose)
	g_message("Parameter file %s not found, try %s/.cora/fit.inp", 
		  ppath, g_get_home_dir());
      g_snprintf(ppath, MAXPATH, "%s/.cora/fit.inp", g_get_home_dir());
    }


  if( (i = read_input (ppath, 0)) < 0)
    {  
      g_warning("Error in parameter file %s.", conffile);  
      exit (3);  
    }

  if (i == 1 || i == 2) /* file not present or wrong version*/
    {
      if (verbose)
	g_message ("Parameter file %s not found, create it.", conffile);
      cora_inp_nu();
      read_input (conffile, 1);
    }
/*  else if ( i == 2)  (wrong version)
      g_warning("Wrong version, setting missing parameters with defaults"); */
  else
    {
	g_snprintf (command, 255, "cp %s %s  2>/dev/null",ppath,conffile);
	system (command);
    }
      

  /* ------------------------------------------------------------------ */

  /*  build GUI                      
   */

  if (verbose) g_message ("Building GUI");

  gtk_init (&argc, &argv);

  separator = gtk_hseparator_new ();
  ttip = gtk_tooltips_new();
 gtk_rc_parse_string("style \"gtk-tooltips-style\" {bg[NORMAL] = \"#ffffc0\"}\n widget \"gtk-tooltips\" style \"gtk-tooltips-style\"");

  if (verbose) g_message ("Creating first window");

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Cora "VERSION);

  /* gtk_widget_set_usize (window, 400, 300); */
  /* gtk_container_resize_children(window); */

  /* When the window is given the "delete_event" signal (this is given
   * by the window manager, usually by the 'close' option, or on the
   * titlebar), we ask it to call the delete_event () function
   * as defined above. The data passed to the callback
   * function is NULL and is ignored in the callback function. 
   */
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);

  /* Here we connect the "destroy" event to a signal handler.  
   * This event occurs when we call gtk_widget_destroy() on the window,
   * or if we return 'FALSE' in the "delete_event" callback. 
   */
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

  /* Sets the border width of the window. */
  /* gtk_container_set_border_width (GTK_CONTAINER (window), 10); */

  /* Create a box to pack widgets into.
   * The box is not really visible, it
   * is just used as a tool to arrange widgets. 
   */
  box_command = gtk_hbox_new (FALSE, 0);
  vbox0 = gtk_vbox_new (FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (vbox0), 10);

  /* Create menubar   
   */
  get_main_menu (window, &menubar);
  gtk_box_pack_start (GTK_BOX (vbox0), menubar, FALSE, TRUE, 0);
  gtk_widget_show (menubar);

  /* Create buttons 
   */
  buttongo   = gtk_button_new_with_label ("GO!");
   gtk_tooltips_set_tip(ttip,buttongo,"Will start fitting", NULL);
  sprintf (labtext, "%s '%s'", " Save actual settings into",conffile);
  buttonsave = gtk_button_new_with_label ("Save Values");
   gtk_tooltips_set_tip(ttip,buttonsave,labtext, NULL);
  buttonquit = gtk_button_new_with_label ("Panic");
   gtk_tooltips_set_tip(ttip,buttonquit,"Quit immediately", NULL);
  /* Instead of gtk_container_add, we pack the buttons into the invisible
   * box, which has been packed into the window. */
  gtk_box_pack_start (GTK_BOX (box_command), buttongo, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (box_command), buttonsave, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (box_command), buttonquit, TRUE, TRUE, 0);

  /* Connect and display buttons 
   */
  gtk_signal_connect (GTK_OBJECT (buttongo), "clicked",
		      GTK_SIGNAL_FUNC (Go), NULL);
  gtk_signal_connect (GTK_OBJECT (buttonsave), "clicked",
		      GTK_SIGNAL_FUNC (make_inputfile), NULL);
  gtk_signal_connect (GTK_OBJECT (buttonquit), "clicked",
		      GTK_SIGNAL_FUNC (gtk_main_quit),
		      NULL);
  gtk_widget_show (buttongo);
  gtk_widget_show (buttonsave);
  gtk_widget_show (buttonquit);

  /* --------------- initialize notebook     ------------------------ */
  
  notebook = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
  
  gtk_box_pack_start (GTK_BOX (vbox0), notebook, TRUE, FALSE, 0);
  
  gtk_widget_show(notebook);
  gtk_widget_show(vbox0);
  
  /* ---------------------------------------------
   *
   *  FIT PARAMETERS
   *
   * --------------------------------------------- */

  label = gtk_label_new ("Fit parameters");
  vbox  = gtk_vbox_new (FALSE, 0);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 
  gtk_widget_show (label);
  gtk_widget_show (vbox);

/*    nlines */
  box_loop = gtk_hbox_new (FALSE, 0);
  sprintf (deftext, "%5d", nlines);
  sprintf (labtext, "%s", "Number of lines");
  entry_arr[2] = create_entry (box_loop, enter_callback, "2",
			       deftext, labtext);
  gtk_tooltips_set_tip(ttip,entry_arr[2],"Number of emission lines", NULL);
  gtk_box_pack_start (GTK_BOX (vbox), box_loop, TRUE, TRUE, 2);

/*    app_s  */
  box_loop = gtk_hbox_new (FALSE, 0);
  sprintf (deftext, "%5.3f", app_s);
  sprintf (labtext, "%s", " Source spectrum aperture");
  create_frame (vbox, box_loop, "Relative Apertures");
  entry_arr[3] = create_entry (box_loop, enter_callback, "3", 
			       deftext, labtext);
   gtk_tooltips_set_tip(ttip,entry_arr[3],"Aperture of source (only ratio with app_b decides)", NULL);

/*    app_b  */
  sprintf (deftext, "%5.3f", app_b);
  sprintf (labtext, "%s", " Background spectrum aperture");
  entry_arr[4] = create_entry (box_loop, enter_callback, "4", 
			       deftext, labtext);
   gtk_tooltips_set_tip(ttip,entry_arr[4],"Aperture of instr. background (only ratio with app_s decides)", NULL);

/*    fixflag  */
  sprintf (deftext, "%5d", fixflag);
  sprintf (labtext, "%s", " fill empty bins");
  toggle[32] = create_toggle (box_loop, enter_switch, "32", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[32],"Set empty bins to zero when spectrum is badly binned", NULL);

/*    sbg   */
  box_loop = gtk_hbox_new (FALSE, 0);
  sprintf (deftext, "%5.2f", sbg);
  sprintf (labtext, "%s", " Estimated source bg per Angstroem");
  create_frame (vbox, box_loop, "Background");
  entry_arr[5] = create_entry (box_loop, enter_callback, "5", 
			       deftext, labtext);
   gtk_tooltips_set_tip(ttip,entry_arr[5],"Value of source background; not used if iterated", NULL);
/*    sbgit */
  hbox  = gtk_vbox_new (FALSE, 0);
  sprintf (deftext, "%5d", sbgit);
  sprintf (labtext, "%s", " Source bg iteration with median");
  toggle[6] = create_toggle (hbox, enter_switch, "6", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[6],"find constant source background with median", NULL);
 /*     medsig  */
  sprintf (deftext, "%5.2f", medsig);
  sprintf (labtext, "%s", " range for median");
  adj = (GtkAdjustment *) gtk_adjustment_new (medsig, 0.1, 11., .1, 1., 0.0);
  spin_arr[2] = create_spin_float (hbox, adj, enter_spin, "33", labtext);
   gtk_tooltips_set_tip(ttip,spin_arr[2],"All bins with counts more then medsig times median will be excluded for final median", NULL);
  gtk_box_pack_start (GTK_BOX (box_loop), hbox, TRUE, FALSE, 0);
  gtk_widget_show (hbox);

/*    sbgchi */
  sprintf (deftext, "%5d", sbgchi);
  sprintf (labtext, "%s", " Source bg iteration with chi");
  toggle[30] = create_toggle (box_loop, enter_switch, "30", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[30],"find constant source background with Chi^2 method", NULL);
/*    bgsmooth */
  sprintf (deftext, "%5d", bgsmooth);
  sprintf (labtext, "%s", " Smoothing of instrumental bg");
  toggle[23] = create_toggle (box_loop, enter_switch, "23", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[23],"smoothe instr. background with polonomial", NULL);

/*    dlam    */
  box_tmp = gtk_vbox_new (FALSE, 0);
  box_loop = gtk_hbox_new (FALSE, 0);
  sprintf (deftext, "%5.3f", dlam);
  sprintf (labtext, "%s", " Range (lambda)");
  entry_arr[9] = create_entry (box_tmp, enter_callback, "9",
                               deftext, labtext);
   gtk_tooltips_set_tip(ttip,entry_arr[9],"Range is used for iteration limits and for the plot range", NULL);

/*    ikev    */
  sprintf (deftext, "%5d", ikev);
  sprintf (labtext, "%s", " ikev");

  box_tmp1 = gtk_hbox_new (FALSE, 0);
  button = gtk_radio_button_new_with_label (NULL, "Angstroem");
   gtk_tooltips_set_tip(ttip,button,"units of x-axis in plot is Angstroem",NULL);
  gtk_box_pack_start (GTK_BOX (box_tmp1), button, TRUE, TRUE, 0);
  if (ikev == 0)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
  gtk_widget_show (button);
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
  toggle[29] = gtk_radio_button_new_with_label (group, "keV");
   gtk_tooltips_set_tip(ttip,toggle[29],"units of x-axis in plot is keV",NULL);
  if (ikev == 1)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle[29]), TRUE);
  gtk_signal_connect (GTK_OBJECT (toggle[29]), "clicked",
                      GTK_SIGNAL_FUNC (enter_switch), "29");
  gtk_widget_show (toggle[29]);
  gtk_box_pack_start (GTK_BOX (box_tmp1), toggle[29], TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box_tmp), box_tmp1, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box_loop), box_tmp, TRUE, FALSE, 0);
  gtk_widget_show (box_tmp);
  gtk_widget_show (box_tmp1);

/*    dsig    */
  box_tmp = gtk_vbox_new (FALSE, 0);
  sprintf (deftext, "%8.5f", dsig);
  sprintf (labtext, "%s", " Range (sigma)");
  create_frame (vbox, box_loop, "Fit ranges");
  entry_arr[7] = create_entry (box_tmp, enter_callback, "7", 
			       deftext, labtext);
   gtk_tooltips_set_tip(ttip,entry_arr[7],"Range is used for iteration limits and for the plot range", NULL);

/*    sigunit   */
  sprintf (deftext, "%5d", sigunit);
  sprintf (labtext, "%s", " sigunit");

  box_tmp1 = gtk_hbox_new (FALSE, 0);
  button = gtk_radio_button_new_with_label (NULL, "Angstroem");
   gtk_tooltips_set_tip(ttip,button,"units of line width will be Angstroem", NULL);
  gtk_box_pack_start (GTK_BOX (box_tmp1), button, TRUE, TRUE, 0);
  if (sigunit == 0)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
  gtk_widget_show (button);
  group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
  toggle[8] = gtk_radio_button_new_with_label (group, "km/sec");
   gtk_tooltips_set_tip(ttip,toggle[8],"units of line width will be km/s (Doppler)", NULL);
  if (sigunit == 1)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle[8]), TRUE);
  gtk_signal_connect (GTK_OBJECT (toggle[8]), "clicked",
		      GTK_SIGNAL_FUNC (enter_switch), "8");
  gtk_widget_show (toggle[8]);

  gtk_box_pack_start (GTK_BOX (box_tmp1), toggle[8], TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box_tmp), box_tmp1, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box_loop), box_tmp, TRUE, FALSE, 0);

  gtk_widget_show (button);
  gtk_widget_show (box_tmp);
  gtk_widget_show (box_tmp1);
  gtk_widget_show (box_loop);

/*    ilamit  */
  box_loop = gtk_hbox_new (FALSE, 0);
  sprintf (deftext, "%5d", ilamit);
  sprintf (labtext, "%s", " Lambda");
  create_frame (vbox, box_loop, "Iterated parameters");
  toggle[10] = create_toggle (box_loop, enter_switch, "10", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[10],"Iterate lambda with Powell method", NULL);

/*    isigit  */
  sprintf (deftext, "%5d", isigit);
  sprintf (labtext, "%s", " Sigma");
  toggle[11] = create_toggle (box_loop, enter_switch, "11", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[11],"Iterate line width with Powell method", NULL);

/*     fixpt  */
  box_loop = gtk_hbox_new (FALSE, 0);
/*  sprintf (deftext, "%5d", iuse_fixpt);
  sprintf (labtext, "%s", "use fixpoint");
  fixtoggle = create_toggle (box_loop, enter_switch, "101", deftext, labtext);
 */

  create_frame (vbox, box_loop, "Iteration options"); 


/*     idlam  */
  sprintf (deftext, "%5d", idlam);
  sprintf (labtext, "%s", "iterate complete multiplet");
  toggle[21] = create_toggle (box_loop, enter_switch, "21", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[21],"Distance between lines is kept constant", NULL);

/*     iord  */
  sprintf (deftext, "%5.2f", iord);
  sprintf (labtext, "%s", " Shift wavelengths by factor");
  adj = (GtkAdjustment *) gtk_adjustment_new (iord, 0.1, 11., .1, 1., 0.0);
  spin_arr[1] = create_spin_float (box_loop, adj, enter_spin, "22", labtext);
   gtk_tooltips_set_tip(ttip,spin_arr[1],"The wavelength of all lines will be multiplied with this number (good for higher dispersion orders or red-shifts)", NULL);

/*     idsig  */
  sprintf (deftext, "%5d", idsig);
  sprintf (labtext, "%s", "iterate an overall sigma");
  toggle[31] = create_toggle (box_loop, enter_switch, "31", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[31],"A constant sigma value is assumed for all lines (the first value which is not zero is taken)", NULL);


  /* ---------------------------------------------
   *
   *  PLOT PARAMETERS
   *
   * --------------------------------------------- */

  label = gtk_label_new ("Output parameters");
  vbox = gtk_vbox_new (TRUE, 0);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
 
  gtk_widget_show (label);
  gtk_widget_show (vbox);

  /* treatment of Fit Title 
   */
  box_loop = gtk_hbox_new (TRUE, 0);
  create_frame (vbox, box_loop, "Title of plot (pgplot-style)");

  sprintf (deftext, "%s", fittitle);
  entry_arr[1] = create_entry (box_loop, enter_callback, "1", deftext, "");
   gtk_tooltips_set_tip(ttip,entry_arr[1],"The title is put on top of the plot", NULL);

  /*    iformat  */

  box_tmp = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (box_tmp);

  button1 = gtk_radio_button_new_with_label (NULL, "X Window");
   gtk_tooltips_set_tip(ttip,button1,"Choose format of plot: X Window", NULL);
  if (iformat == 0)
    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button1), TRUE);
  gtk_signal_connect (GTK_OBJECT (button1), "toggled",
		      GTK_SIGNAL_FUNC (enter_of), "0");
  gtk_box_pack_start (GTK_BOX (box_tmp), button1, TRUE, FALSE, 0);
  gtk_widget_show (button1);

  groupx = gtk_radio_button_group (GTK_RADIO_BUTTON (button1));
  button2 = gtk_radio_button_new_with_label (groupx, "Postscript");
   gtk_tooltips_set_tip(ttip,button2,"Choose format of plot: Ps file", NULL);
  if (iformat == 1)
    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button2), TRUE);
  gtk_signal_connect (GTK_OBJECT (button2), "toggled",
		      GTK_SIGNAL_FUNC (enter_of), "1");
  gtk_box_pack_start (GTK_BOX (box_tmp), button2, TRUE, FALSE, 0);
  gtk_widget_show (button2);

  groupx = gtk_radio_button_group (GTK_RADIO_BUTTON (button2));
  button3 = gtk_radio_button_new_with_label (groupx, "GIF");
   gtk_tooltips_set_tip(ttip,button3,"Choose format of plot: GIF file", NULL);
  if (iformat == 2)
    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button3), TRUE);
  gtk_signal_connect (GTK_OBJECT (button3), "toggled",
		      GTK_SIGNAL_FUNC (enter_of), "2");
  gtk_box_pack_start (GTK_BOX (box_tmp), button3, TRUE, FALSE, 0);
  gtk_widget_show (button3);

  groupx = gtk_radio_button_group (GTK_RADIO_BUTTON (button3));
  button4 = gtk_radio_button_new_with_label (groupx, "PPM");
   gtk_tooltips_set_tip(ttip,button4,"Choose format of plot: PPM file", NULL);
  if (iformat == 3)
    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button4), TRUE);
  gtk_signal_connect (GTK_OBJECT (button4), "toggled",
		      GTK_SIGNAL_FUNC (enter_of), "3");
  gtk_box_pack_start (GTK_BOX (box_tmp), button4, TRUE, FALSE, 0);
  gtk_widget_show (button4);

/*    iportrait  */
  box_loop = gtk_hbox_new (TRUE, 0);
  sprintf (deftext, "%5d", iportrait);
  sprintf (labtext, "%s", " Orientation portrait");
  toggle[24] = create_toggle (box_loop, enter_switch, "24", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[24],"Choose orientation of plot (Selected means portrait)", NULL);

  box_tmp1 = gtk_vbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (box_tmp1), box_tmp, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box_tmp1), box_loop, TRUE, FALSE, 0);
  gtk_widget_show (box_tmp1);

  create_frame (vbox, box_tmp1, "Output format");

/*    iplot */
  box_loop = gtk_hbox_new (TRUE, 0);
  sprintf (deftext, "%5d", iplot);
  sprintf (labtext, "%s", " Create plot");
  create_frame (vbox, box_loop, "Plot parameters");
  toggle[14] = create_toggle (box_loop, enter_switch, "14", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[14],"Plotting can be suppressed by deactivating", NULL);

/*    ilab   */
  sprintf (deftext, "%5d", ilab);
  sprintf (labtext, "%s", " Include labels");
  toggle[15] = create_toggle (box_loop, enter_switch, "15", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[15],"Put labels into plot: Fit results at proper position, list of iterated variables", NULL);

/*    ierrplot */
  sprintf (deftext, "%5d", ierrplot);
  sprintf (labtext, "%s", " Plot error bars");
  toggle[16] = create_toggle (box_loop, enter_switch, "16", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[16],"Include error bars", NULL);

/*    ilog    */
  sprintf (deftext, "%5d", ilog);
  sprintf (labtext, "%s", " Logarithmic plot");
  toggle[17] = create_toggle (box_loop, enter_switch, "17", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[17],"Plot with logarithmic y axis", NULL);

/*    nbin  */
  sprintf (deftext, "%5d", nbin);
  sprintf (labtext, "%s", " Bin data");
  adj = (GtkAdjustment *) gtk_adjustment_new (nbin, 1, 20, 1, 1, 0);
  spin_arr[0] = create_spin (box_loop, adj, enter_spin, "18", labtext);
   gtk_tooltips_set_tip(ttip,spin_arr[0],"Choose number of original bins comprised in one new bin (1 is best resolution)", NULL);

  box_tmp1 = gtk_vbox_new (TRUE, 0);
/*    spec_only   */
  sprintf (deftext, "%5d", spec_only);
  sprintf (labtext, "%s", " suppress plot of fit");
  toggle[26] = create_toggle (box_tmp1, enter_switch, "26", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[26],"Plot only the spectrum, do no iteration. Plot ranges are determined from the wavelengths and the ranges for lambda and sigma", NULL);

/*    fit_only   */
  sprintf (deftext, "%5d", fit_only);
  sprintf (labtext, "%s", " suppress plot of spectrum");
  toggle[27] = create_toggle (box_tmp1, enter_switch, "27", deftext, labtext);
  gtk_box_pack_start (GTK_BOX (box_loop), box_tmp1, TRUE, FALSE, 0);
   gtk_tooltips_set_tip(ttip,toggle[27],"Plot only the theoretical curve, but not the measured spectrum", NULL);

/*     nlik
  box_loop = gtk_hbox_new (TRUE, 0);
  sprintf (deftext, "%5d", nlik);
  sprintf (labtext, "%s", " Number of data points");
  create_frame (vbox, box_loop, 
		"Plot likelihood curve (if number of data points > 0)");
  adj = (GtkAdjustment *) gtk_adjustment_new (nlik, 0, 1000, 10, 1, 0);
  spin_arr[2] = create_spin (box_loop, adj, enter_spin, "19", labtext);

     ispline  
  sprintf (deftext, "%5d", ispline);
  sprintf (labtext, "%s", " Use spline interpolation");
  toggle[20] = create_toggle (box_loop, enter_switch, "20", deftext, labtext);
 */

/*    ires  */
  box_loop = gtk_hbox_new (FALSE, 0);
  sprintf (deftext, "%5d", ires);
  sprintf (labtext, "%s '%s'", " Print results into file",resfile);
  create_frame (vbox, box_loop, "Log output");
  toggle[13] = create_toggle (box_loop, enter_switch, "13", deftext, labtext);
   gtk_tooltips_set_tip(ttip,toggle[13],"This option is intended for creating a log file of all actions; the file can be used for creating a TeX table", NULL);

/*    verbosity  */
  sprintf (deftext, "%d", verbose);
  sprintf (labtext, "%s ", " degree of verbosity");
  adj = (GtkAdjustment *) gtk_adjustment_new (verbose, 0, 3, 1, 1, 0);
  spin_arr[2] = create_spin (box_loop, adj, enter_spin, "25", labtext);
   gtk_tooltips_set_tip(ttip,spin_arr[2],"Three levels of verbosity can be choosen for following the run of the program (debugging or just beeing curious)", NULL);

  /* ---------------------------------------------
   *
   *  FOOT
   *
   * --------------------------------------------- */



  /* Put vbox0 into the window 
   */
  gtk_container_add (GTK_CONTAINER (window), vbox0);

  /* Put the command box into the vertical box. */

  box_loop = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box_loop), separator, FALSE, TRUE, 5);
  gtk_widget_show (separator);
  gtk_widget_show (box_loop);
  gtk_box_pack_start (GTK_BOX (vbox0), box_loop, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox0), box_command, TRUE, TRUE, 0);
  gtk_widget_show (box_command);
  /* gtk_widget_show (vbox); */

  /* The final step is to display the newly created widgets. */
  /* and the window */
  gtk_widget_show (window);

  if (count == 0)
   {
    popup_message("No spectrum read in !");
    spec_sel();
   }

  /* All GTK applications must have a gtk_main(). Control ends here
   * and waits for an event to occur (like a key press or mouse event). */
  gtk_main ();

  return (0);
}

